#!/usr/bin/python3

import base64
import hmac
import struct
import sys
import time


def hotp(secret, counter, digits=6, digest='sha1'):
    padding = '=' * ((8 - len(secret)) % 8)
    secret_bytes = base64.b32decode(secret.upper() + padding)
    counter_bytes = struct.pack(">Q", counter)
    mac = hmac.new(secret_bytes, counter_bytes, digest).digest()
    offset = mac[-1] & 0x0f
    truncated = struct.unpack('>L', mac[offset:offset+4])[0] & 0x7fffffff
    return str(truncated)[-digits:].rjust(digits, '0')


def totp(secret, interval=30):
    return hotp(secret, int(time.time() / interval))


def main():
    for secret in sys.stdin:
        print(totp(secret.strip()))


if __name__ == '__main__':
    main()



import setuptools


def long_desc():
    readme = open('README.md').read()
    i = readme.index('[')
    j = readme.index('\nIntroduction')
    k = readme.index('\n\nSource Code')
    m = readme.index('\n\nResources')
    n = readme.index('\n\nLicense')
    return readme[:i] + readme[j:k] + readme[m:n]


setuptools.setup(
    name='mintotp',
    version='0.1.0',
    author='Susam Pal',
    author_email='susam@susam.in',
    description='MinTOTP - Minimal TOTP Generator',
    long_description=long_desc(),
    long_description_content_type='text/markdown',
    url='https://github.com/susam/mintotp',

    py_modules=['mintotp'],
    entry_points={
        'console_scripts': {
            'mintotp = mintotp:main'
        }
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
    ],

    keywords='totp hotp otp hmac cryptography 2fa authenticator',
)
